package cz.muni.pa165.car.rest;

import cz.muni.pa165.car.service.CarComponentPairService;
import cz.muni.pa165.common_library.dtos.CarComponentDto;
import cz.muni.pa165.common_library.dtos.CarDto;
import io.swagger.v3.oas.annotations.Operation;
import java.util.List;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Rest Controller for Driver Manager.
 */
@RestController
@RequestMapping(path = "/carcomponent")
@Validated
public class CarComponentPairController {

  CarComponentPairService carComponentService;

  public CarComponentPairController(CarComponentPairService carComponentService) {
    this.carComponentService = carComponentService;
  }

  /**
   * Calls service to add a component to a car.
   *
   * @param componentId Id of the component.
   * @param carId       Id of the car.
   * @return Car entity.
   */
  @Operation(summary = "Add component to a car")
  @PutMapping(path = "/addcomponent",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarDto> addComponent(@RequestParam Long componentId,
                                             @RequestParam Long carId) {
    return ResponseEntity.ok(carComponentService.addComponent(componentId, carId));
  }

  /**
   * Calls service to remove a component from a car.
   *
   * @param componentId Id of the component.
   * @param carId       Id of the car.
   * @return Car entity.
   */
  @Operation(summary = "Remove component from a car")
  @PutMapping(path = "/removecomponent",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarDto> removeComponent(@RequestParam Long componentId,
                                                @RequestParam Long carId) {
    return ResponseEntity.ok(carComponentService.removeComponent(componentId, carId));
  }

  /**
   * Calls service to get all components of a car.
   *
   * @param carId Id of the car.
   * @return List of components.
   */
  @Operation(summary = "Get all components of a car")
  @PutMapping(path = "/getcomponents",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<List<CarComponentDto>> getAllComponentsOfCar(@RequestParam Long carId) {
    return ResponseEntity.ok(carComponentService.getAllComponentsOfCar(carId));
  }


}
