package cz.muni.pa165.car.rest;

import cz.muni.pa165.car.service.CarDriverPairService;
import cz.muni.pa165.common_library.dtos.CarDto;
import cz.muni.pa165.common_library.dtos.DriverDto;
import io.swagger.v3.oas.annotations.Operation;
import java.util.List;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Rest Controller for Driver Manager.
 */
@RestController
@RequestMapping(path = "/cardriver")
@Validated
public class CarDriverPairController {

  CarDriverPairService carDriverService;

  public CarDriverPairController(CarDriverPairService driverManagerService) {
    this.carDriverService = driverManagerService;
  }

  /**
   * Calls service to assign a driver to a car.
   *
   * @param driverId Id of the driver.
   * @param carId    Id of the car.
   * @return Car entity.
   */
  @Operation(summary = "Assign driver to a car")
  @PutMapping(path = "/assign",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarDto> assignDriverToCar(@RequestParam Long driverId,
                                                  @RequestParam Long carId) {
    return ResponseEntity.ok(carDriverService.assignDriverToCar(driverId, carId));
  }

  /**
   * Calls service to unassing a driver from a car.
   *
   * @param driverId Id of the driver.
   * @param carId    Id of the car.
   * @return Car entity.
   */
  @Operation(summary = "Unassign driver from a car")
  @PutMapping(path = "/unassign",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarDto> unassignDriverFromCar(@RequestParam Long driverId,
                                                      @RequestParam Long carId) {
    return ResponseEntity.ok(carDriverService.unassignDriverFromCar(driverId, carId));
  }

  /**
   * Calls service to return all drivers assigned to a car.
   *
   * @param carId Id of the car.
   * @return List of drivers.
   */
  @Operation(summary = "Return all drivers of the car")
  @PutMapping(path = "/alldrivers",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<List<DriverDto>> getAllDriversOfCar(@RequestParam Long carId) {
    return ResponseEntity.ok(carDriverService.getAllDriversOfCar(carId));
  }
}
