package cz.muni.pa165.car.service;

import cz.muni.pa165.car.data.repository.CarComponentPairRepository;
import cz.muni.pa165.car.data.repository.CarComponentRepository;
import cz.muni.pa165.car.data.repository.CarRepository;
import cz.muni.pa165.common_library.dtos.CarComponentDto;
import cz.muni.pa165.common_library.dtos.CarDto;
import cz.muni.pa165.common_library.exception.DatabaseException;
import cz.muni.pa165.component.data.model.CarComponent;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import org.springframework.stereotype.Service;

/**
 * Service for manipulation with car's components.
 */
@Service
public class CarComponentPairServiceImpl implements CarComponentPairService {

  CarComponentPairRepository carComponentPairRepository;
  CarComponentRepository carComponentRepository;
  CarRepository carRepository;

  /**
   * Constructor for Car - Car Component Service.
   *
   * @param carComponentPairRepository Car - Car Component repository
   * @param carComponentRepository Car Component repository
   * @param carRepository Car repository
   */
  public CarComponentPairServiceImpl(CarComponentPairRepository carComponentPairRepository,
                                     CarComponentRepository carComponentRepository,
                                     CarRepository carRepository) {
    this.carComponentPairRepository = carComponentPairRepository;
    this.carComponentRepository = carComponentRepository;
    this.carRepository = carRepository;
  }

  @Override
  public CarDto addComponent(Long componentId, Long carId) {
    var car = carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException("Car not found"));
    var component = carComponentRepository.findById(componentId).orElseThrow(
        () -> new DatabaseException("Component not found"));
    var components = car.getComponents();
    components.add(component);
    car.setComponents(components);
    carRepository.save(car);
    return CarServiceImpl.carConverterToDto(car);
  }

  @Override
  public CarDto removeComponent(Long componentId, Long carId) {
    var car = carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException("Car not found"));
    var components = new ArrayList<CarComponent>();
    for (CarComponent d : car.getComponents()) {
      if (!Objects.equals(d.getId(), componentId)) {
        components.add(d);
      }
    }
    car.setComponents(components);
    carRepository.save(car);
    return CarServiceImpl.carConverterToDto(car);
  }

  @Override
  public List<CarComponentDto> getAllComponentsOfCar(Long carId) {
    var car = carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException("Car not found"));
    var componentDtos = new ArrayList<CarComponentDto>();
    for (CarComponent c : car.getComponents()) {
      componentDtos.add(
          new CarComponentDto(
              c.getId(),
              c.getWeight(),
              c.getPrice(),
              c.getManufacturer(),
              c.getName()
          )
      );
    }
    return componentDtos;
  }

}
