package cz.muni.pa165.car.service;

import cz.muni.pa165.common_library.dtos.CarDto;
import cz.muni.pa165.common_library.dtos.DriverDto;
import java.util.List;

/**
 * Interface fot assign operations of drivers and cars.
 */
public interface CarDriverPairService {
  /**
   * Calls repository to assign a driver to a car, if unsuccessful, throws exception.
   *
   * @param driverId Id of the diver.
   * @param carId    Id of the car.
   * @return Car object that the driver was assigned to.
   */
  CarDto assignDriverToCar(Long driverId, Long carId);

  /**
   * Calls repository to unassign a driver from a car, if unsuccessful, throws exception.
   *
   * @param driverId Id of the diver.
   * @param carId    Id of the car.
   * @return Car object that the driver was unassigned from.
   */
  CarDto unassignDriverFromCar(Long driverId, Long carId);

  /**
   * Calls repository to return list of all drivers of specific car.
   *
   * @param carId Id of the car.
   * @return List of all drivers assigned to the car.
   */
  List<DriverDto> getAllDriversOfCar(Long carId);
}
