package cz.muni.pa165.car.service;

import cz.muni.pa165.car.data.repository.CarDriverPairRepository;
import cz.muni.pa165.car.data.repository.CarRepository;
import cz.muni.pa165.car.data.repository.DriverRepository;
import cz.muni.pa165.common_library.dtos.CarDto;
import cz.muni.pa165.common_library.dtos.DriverDto;
import cz.muni.pa165.common_library.exception.DatabaseException;
import cz.muni.pa165.driver.data.model.Driver;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import org.springframework.stereotype.Service;

/**
 * Service for Driver Manager.
 */
@Service
public class CarDriverPairServiceImpl implements CarDriverPairService {

  CarDriverPairRepository carDriverPairRepository;
  DriverRepository driverRepository;
  CarRepository carRepository;

  /**
   * Constructor for Car - Driver Service.
   *
   * @param carDriverPairRepository Car - Driver repository
   * @param driverRepository Driver repository
   * @param carRepository Car repository
   */
  public CarDriverPairServiceImpl(CarDriverPairRepository carDriverPairRepository,
                                  DriverRepository driverRepository,
                                  CarRepository carRepository) {
    this.carDriverPairRepository = carDriverPairRepository;
    this.driverRepository = driverRepository;
    this.carRepository = carRepository;
  }

  @Override
  public CarDto assignDriverToCar(Long driverId, Long carId) {
    var car = carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException("Car not found"));
    var driver = driverRepository.findById(driverId).orElseThrow(
        () -> new DatabaseException("Driver not found"));
    var drivers = car.getDrivers();
    drivers.add(driver);
    car.setDrivers(drivers);
    carRepository.save(car);
    return CarServiceImpl.carConverterToDto(car);
  }

  @Override
  public CarDto unassignDriverFromCar(Long driverId, Long carId) {
    var car = carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException("Car not found"));
    var drivers = new ArrayList<Driver>();
    for (Driver d : car.getDrivers()) {
      if (!Objects.equals(d.getId(), driverId)) {
        drivers.add(d);
      }
    }
    car.setDrivers(drivers);
    carRepository.save(car);
    return CarServiceImpl.carConverterToDto(car);
  }

  @Override
  public List<DriverDto> getAllDriversOfCar(Long carId) {
    var car = carRepository.findById(carId).orElseThrow(
        () -> new DatabaseException("Car not found"));
    var driverDtos = new ArrayList<DriverDto>();
    for (Driver d : car.getDrivers()) {
      driverDtos.add(
          new DriverDto(d.getId(),
              d.getName(),
              d.getSurname())
      );
    }
    return driverDtos;
  }

}
