package cz.muni.pa165.common_library.validator;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;
import javax.validation.Constraint;
import javax.validation.Payload;

/**
 * This annotation is used to validate that the provided map of characteristics
 * contains only valid characteristics, i.e. characteristics that are specified
 * in the list of allowed characteristics provided as an argument to the
 * annotation.
 */
@Target({FIELD})
@Retention(RUNTIME)
@Constraint(validatedBy = CharacteristicsValidator.class)
public @interface ValidCharacteristics {

  /**
   * The list of allowed characteristics.
   *
   * @return the list of allowed characteristics.
   */
  String[] value();

  /**
   * The error message to be used if the validation fails.
   *
   * @return the error message.
   */
  String message() default "Invalid characteristic";

  /**
   * The validation group(s) this constraint belongs to.
   *
   * @return the validation group(s).
   */
  Class<?>[] groups() default {};

  /**
   * Payload to be attached to the constraint.
   *
   * @return the payload.
   */
  Class<? extends Payload>[] payload() default {};
}

