package cz.muni.pa165.component.rest;

import cz.muni.pa165.common_library.dtos.CarComponentDto;
import cz.muni.pa165.component.service.ComponentServiceInterface;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import java.util.List;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Controller for managing the components.
 */
@RestController
@RequestMapping(path = "/component")
@Validated
public class ComponentController {

  ComponentServiceInterface componentService;

  ComponentController(ComponentServiceInterface componentFacade) {
    this.componentService = componentFacade;
  }

  @Operation(summary = "Create new component")
  @PostMapping(path = "/", consumes = MediaType.APPLICATION_JSON_VALUE,
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarComponentDto> createComponent(
      @Valid @RequestBody CarComponentDto carComponent) {
    return ResponseEntity.ok(componentService.postCarComponent(carComponent));
  }

  @Operation(summary = "Delete a car component")
  @DeleteMapping(path = "/",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<String> deleteComponent(@Valid @RequestParam long componentId) {
    return ResponseEntity.ok(componentService.deleteById(componentId));
  }

  @Operation(summary = "Get a car component")
  @GetMapping(path = "/id",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<CarComponentDto> getComponent(@Valid @RequestParam long componentId) {
    return ResponseEntity.ok(componentService.getCarComponentById(componentId));
  }

  @Operation(summary = "Get all car components")
  @GetMapping(path = "/",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<List<CarComponentDto>> getComponents() {
    return ResponseEntity.ok(componentService.getAllCarComponents());
  }
}
