package cz.muni.pa165.component.service;

import cz.muni.pa165.common_library.dtos.CarComponentDto;
import cz.muni.pa165.common_library.exception.DatabaseException;
import cz.muni.pa165.component.data.model.CarComponent;
import cz.muni.pa165.component.data.repository.ComponentRepositoryInterface;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


/**
 * Service for component initialization.
 */
@Service
public class ComponentService implements ComponentServiceInterface {

  @Autowired
  ComponentRepositoryInterface componentRepository;

  ComponentService(ComponentRepositoryInterface componentRepository) {
    this.componentRepository = componentRepository;
  }

  public CarComponentDto postCarComponent(CarComponentDto carComponentDto) {
    return carComponentConverter(componentRepository.save(
        carComponentDtoConverter(carComponentDto)));
  }

  /**
   * Finds and returns car component by id.
   *
   * @param carComponentId id of the component.
   * @return found car component.
   */
  public CarComponentDto getCarComponentById(Long carComponentId) {
    return carComponentConverter(componentRepository.findById(carComponentId).orElseThrow(
        () -> new DatabaseException("Something went wrong when finding car component with id: "
            + carComponentId + " in the database.")));
  }

  /**
   * Get all the car components in the repository.
   *
   * @return list of stored car components.
   */
  public List<CarComponentDto> getAllCarComponents() {
    return componentRepository.findAll()
        .stream()
        .map(this::carComponentConverter)
        .toList();
  }

  /**
   * Removes car component from repository based on the id.
   *
   * @param carComponentId of the car component for removal.
   */
  public String deleteById(Long carComponentId) {
    componentRepository.deleteById(carComponentId);
    return "Car component with id: " + carComponentId + "was successfully deleted";
  }

  CarComponent carComponentDtoConverter(CarComponentDto carComponentDto) {
    return CarComponent.builder()
        .id(carComponentDto.getId())
        .weight(carComponentDto.getWeight())
        .price(carComponentDto.getPrice())
        .manufacturer(carComponentDto.getManufacturer())
        .name(carComponentDto.getName())
        .build();
  }

  CarComponentDto carComponentConverter(CarComponent carComponent) {
    return CarComponentDto.builder()
        .id(carComponent.getId())
        .weight(carComponent.getWeight())
        .price(carComponent.getPrice())
        .manufacturer(carComponent.getManufacturer())
        .name(carComponent.getName())
        .build();
  }
}
