package cz.muni.pa165.driver.data.model;

import jakarta.persistence.CollectionTable;
import jakarta.persistence.ElementCollection;
import jakarta.persistence.Entity;
import jakarta.persistence.FetchType;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.MapKeyColumn;
import jakarta.persistence.Table;
import jakarta.validation.constraints.Max;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * Driver Class.
 */
@Data
@Builder
@Entity
@NoArgsConstructor
@AllArgsConstructor
@Table(name = "driver")
public class Driver implements Serializable {

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Long id;

  @NotNull
  @NotBlank
  private String name;

  @NotNull
  @NotBlank
  private String surname;

  @NotNull
  @NotBlank
  private String nationality;

  @NotNull
  @ElementCollection(fetch = FetchType.EAGER)
  @CollectionTable(name = "characteristics")
  @MapKeyColumn(name = "ch_name")
  private Map<String, @Min(0) @Max(5) Integer> characteristics;

  /**
   * Basic constructor when initializing driver.
   *
   * @param name driver name
   * @param surname driver surname
   * @param nationality driver nationality
   */
  public Driver(String name, String surname, String nationality) {
    this.name = name;
    this.surname = surname;
    this.nationality = nationality;;
    this.characteristics = new HashMap<>() {
      {
        put("Experience", 1);
        put("Aggressiveness", 1);
        put("Consistency", 1);
        put("Racecraft", 1);
      }};
  }
}

