package cz.muni.pa165.driver.mapper;

import cz.muni.pa165.common_library.dtos.DriverAddDto;
import cz.muni.pa165.common_library.dtos.DriverInsightDto;
import cz.muni.pa165.common_library.dtos.DriverResponseDto;
import cz.muni.pa165.driver.data.model.Driver;
import org.springframework.stereotype.Component;

/**
 * Implementation of driver mapper.
 */
@Component
public class DriverMapperImpl implements DriverMapper {

  /**
   * Maps driver class to driver response dto.
   *
   * @param driver driver class
   * @return driver response dto
   */
  @Override
  public DriverResponseDto convertToResponseDto(Driver driver) {
    return new DriverResponseDto(driver.getId());
  }

  /**
   * Maps driver class to driver insight dto.
   *
   * @param driver driver class
   * @return driver insight dto
   */
  @Override
  public DriverInsightDto convertToInsightDto(Driver driver) {
    return new DriverInsightDto(
        driver.getId(),
        driver.getName(),
        driver.getSurname(),
        driver.getNationality(),
        driver.getCharacteristics());
  }

  /**
   * Maps driver dto to driver class.
   *
   * @param driverAddDto driver dto
   * @return driver class
   */
  @Override
  public Driver convertToDriver(DriverAddDto driverAddDto) {
    var driver = new Driver(
        driverAddDto.name(),
        driverAddDto.surname(),
        driverAddDto.nationality()
    );

    // Overwrites default characteristics levels with new ones
    if (driverAddDto.characteristics() != null) {
      driver.getCharacteristics().putAll(driverAddDto.characteristics());
    }

    return driver;
  }
}
