package cz.muni.pa165.race.rest;


import cz.muni.pa165.common_library.dtos.RaceDto;
import cz.muni.pa165.race.service.RaceService;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import java.util.List;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Controller for managing races.
 */
@RestController
@RequestMapping(path = "race/")
@Validated
public class RaceController {

  RaceService raceService;

  public RaceController(RaceService raceService) {
    this.raceService = raceService;
  }

  @Operation(summary = "Create a race")
  @PostMapping(path = "/", consumes = MediaType.APPLICATION_JSON_VALUE,
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<RaceDto> createRace(@Valid @RequestBody RaceDto race) {
    return ResponseEntity.ok(raceService.postRace(race));
  }

  @Operation(summary = "Delete a race")
  @DeleteMapping(path = "/",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<String> deleteRace(@Valid @RequestParam Long raceId) {
    return ResponseEntity.ok(raceService.deleteRace(raceId));
  }

  @Operation(summary = "Get a race")
  @GetMapping(path = "/id",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<RaceDto> getRace(@Valid @RequestParam Long raceId) {
    return ResponseEntity.ok(raceService.findRaceById(raceId));
  }

  @Operation(summary = "Get all races")
  @GetMapping(path = "/",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<List<RaceDto>> getRaces() {
    return ResponseEntity.ok(raceService.findRaces());
  }

  @Operation(summary = "Assign driver as driver number one for a race")
  @PatchMapping(path = "/assignDriverOne",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<RaceDto> assignDriverOne(@RequestParam Long driverOneId,
                                                @RequestParam Long raceId,
                                                @RequestParam Long carId) {
    return ResponseEntity.ok(raceService.assignDriverOne(driverOneId, raceId, carId));
  }

  @Operation(summary = "Assign driver as a driver number two for a race")
  @PatchMapping(path = "/assignDriverTwo",
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<RaceDto> assignDriverTwo(@RequestParam Long driverTwoId,
                                                @RequestParam Long raceId,
                                                @RequestParam Long carId) {
    return ResponseEntity.ok(raceService.assignDriverTwo(driverTwoId, raceId, carId));
  }
}
