package cz.muni.pa165.race.rest;

import cz.muni.pa165.common_library.dtos.SeasonDto;
import cz.muni.pa165.race.service.SeasonServiceInterface;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import java.util.List;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Controller for managing seasons.
 */
@RestController
@RequestMapping(path = "/season")
@Validated
public class SeasonController {

  SeasonServiceInterface seasonService;

  public SeasonController(SeasonServiceInterface seasonService) {
    this.seasonService = seasonService;
  }

  @Operation(summary = "Create a season")
  @PostMapping(path = "/", consumes = MediaType.APPLICATION_JSON_VALUE,
      produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<SeasonDto> createSeason(@Valid @RequestBody SeasonDto season) {
    return ResponseEntity.ok(seasonService.postSeason(season));
  }

  @Operation(summary = "Get a season")
  @GetMapping(path = "/id", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<SeasonDto> getSeason(@RequestParam long seasonId) {
    return ResponseEntity.ok(seasonService.getSeasonById(seasonId));
  }

  @Operation(summary = "Get all seasons")
  @GetMapping(path = "/")
  public ResponseEntity<List<SeasonDto>> getAllSeasons() {
    return ResponseEntity.ok(seasonService.getAllSeasons());
  }

  @Operation(summary = "Delete a season")
  @DeleteMapping(path = "/", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<String> deleteSeason(@Valid @RequestParam long seasonId) {
    return ResponseEntity.ok(seasonService.deleteById(seasonId));
  }

  @Operation(summary = "Add a race to a season")
  @PatchMapping(path = "/addRace", produces = MediaType.APPLICATION_JSON_VALUE)
  public ResponseEntity<SeasonDto> addRace(@Valid @RequestParam long seasonId,
                                           @Valid @RequestParam long raceId) {
    return ResponseEntity.ok(seasonService.addRace(seasonId, raceId));
  }
}
